module OrientDb
  class LearningObjectRepository < Base
    include OrientDb::Methods::SocialMethods
    include OrientDb::Methods::EdgeMethods
    include OrientDb::Methods::CountableMethods
    include RepositoriesProxy

    #get the ten most visualised. It's probably making too many useless accesses to orientDB...
    def get_most_visualised
      result = connection.query("SELECT FROM (SELECT @rid,in('Views').size() AS views FROM LearningObject) ORDER BY views DESC LIMIT 10")
      most_visualised = []
      result.each do |result|
        most_visualised.push(get_by_rid(result["rid"]))
      end
      build_objects most_visualised
    end

    def get_publisher(learning_object)
      result = get_edges_end("PublishedBy", "out", learning_object.id)
      result = result.first

      unless result.blank?
        if result["@class"] == "Institution"
          institution_repository.build_object(result)
        else
          user_repository.build_object(result)
        end
      end
    end

    # Usage:
    #   learning_object = repository.for(:learning_objects).get_by_dspace_id 123
    #
    def get_by_dspace_id(id_dspace)
      result = select_by_property(odb_class, "id_dspace", id_dspace)
      build_object result.first
    end

    #all method returns all learning objects of an user
    def all(user)
      build_objects(get_edges_end('PublishedBy', 'in', user.rid) || [])
    end

    def all_by_score_from_offset_to_limit(offset, limit)
      return [] if offset.blank? || limit.blank?

      # expensive query
      key = 'learning_objects/all/score/' + offset.to_s + '/' + limit.to_s
      objects_hash = Rails.cache.fetch(key, expires_in: 6.hours) do
        objects = connection.query "SELECT FROM LearningObject ORDER BY score DESC LIMIT #{limit} SKIP #{offset}", limit: limit
        objects.map { |o| {'rid' => o['@rid']} }
      end

      LearningObjectBuilder.build(objects_hash)
    end

    def get_subjects(learning_object)
      result = get_edges_end("IsAbout", "out", learning_object.id)
      subject_repository.build_objects(result)
    end

    def get_attributes(learning_object)
      attribute_repository.find_all_by_learning_object learning_object
    end

    def create_relations(learning_object)
      relations = OrientDb::LearningObject::RelationsChain.new(learning_object, connection)
      relations.create
    end

    def save(learning_object)
      if learning_object.valid?
        connection.command "INSERT INTO LearningObject CONTENT #{learning_object.to_json}"
      end
    end

    def types
      Rails.cache.fetch("learning_object/types", expires_in: 1.days) do
        query = "select type from LearningObject GROUP BY type"
        types_hash = connection.query query, limit: 100
        # return array with types
        types_hash.map do |e|
          e['type']
        end
      end
    end

    ##
    # To create "index:learningobject_search" on OrientDB, use the following command:
    #   CREATE INDEX learningobject_search
    #   ON LearningObject (name, description)
    #   FULLTEXT ENGINE LUCENE
    #   METADATA {"analyzer":"org.apache.lucene.analysis.br.BrazilianAnalyzer"}
    def search(params)
      cache_key = build_search_cache_key params

      # get results in cache or search
      Rails.cache.fetch(cache_key, expires_in: 10.minutes) do
        query = build_search_query params
        connection.query query, limit: 10000
      end
    end

    def build_object(args={})
      LearningObjectBuilder.build_from_orientdb args
    end

    def build_hash(object)
      hash = super(object)
      # Remove relation attributes
      hash.delete("likes")
      hash.delete("views")
      hash.delete("downloads")
      hash.delete("subjects")
      hash.delete("attributes")
      hash.delete("publisher")
      hash
    end

    def fetch_year(years)
      if years.size() == 1
        year = years[0]
        qry = "created_at.format('yyyy') = '#{year}'"
      elsif years.size() == 2
        begin_year = years[0]
        end_year = years[1]
        qry = "created_at.format('yyyy') >= '#{begin_year}' AND created_at.format('yyyy') <= '#{end_year}'"
      end
    end

    def fetch_types(types)
      qry = ''
      types.each do |type|
        qry = qry + "type = '#{type}' " if type == types.first
        qry = qry + "OR type = '#{type}' " if type != types.first
      end
      qry
    end

    private

    def order_by(order)
      case order
        when 'author'
          'author'
        when 'publicationasc'
          'published_at'
        when 'publicationdesc'
          'published_at DESC'
        when 'title'
          'name ASC'
        else
          'score DESC'
      end
    end

    def accepted_properties
      ['thumbnail', 'score']
    end

    def build_search_query(params={})
      query = "SELECT FROM LearningObject WHERE [name, description, author] LUCENE '#{params[:query]}'"
      query += " AND out('IsAbout') CONTAINS (name in ['" + params[:subject].join("','") + "'])" unless params[:subject].blank?
      unless params[:school_level].blank?
        params[:school_level] << 'Ensino Fundamental Inicial' << 'Ensino Fundamental Final' if params[:school_level].include? 'Ensino Fundamental'
        query += " AND out('IsAbout') CONTAINS (name in ['" + params[:school_level].join("','") + "'])"
      end
      query = "SELECT * FROM (" + query + ") WHERE " + fetch_types(params[:type]) unless params[:type].blank?
      query = "SELECT * FROM (" + query + ") WHERE " + fetch_year(params[:year]) unless params[:year].blank?
      query = "SELECT * FROM (" + query + ") ORDER BY #{order_by(params[:order])}"
      query = "SELECT @rid.asString(), last_modified FROM (" + query + ")"
      query
    end

    def search_params
      %w(query order subject type school_level year)
    end

    def build_search_cache_key(params={})
      cache_key = "search_result/"
      search_params.each do |param|
        if params[param].class == Array
          cache_key += params[param].join('-')
        else
          cache_key += params[param].to_s
        end
        cache_key += '/' unless (param == search_params.last)
      end
      cache_key
    end

  end
end
