module OrientDb
  module Methods
    module SocialMethods
      include OrientDb::Methods::EdgeMethods
      include OrientDb::Methods::SanitizableMethods

      ##
      # This methods destroy a 'like' relation between +user+ and +object+
      # If the +user+ hasn't liked the +object+ throw an exception.
      def dislike!(user, object)
        if liked? user, object
          dislike user, object
        end
        raise 'The user hasn`t likes this object.'
      end

      ##
      # This methods destroy a 'like' relation between +user+ and +object+
      # If the +user+ hasn't liked the +object+ nothing happens
      # If you want raise an exeception, use the LearningObjectRepository::dislike! method.
      def dislike(user, object)
        like_edge_class = 'Likes'
        if liked? user, object
          destroy_edge like_edge_class, user.rid, object.id
          object.likes = object.likes - 1
          true
        end
      end

      ##
      # Call this method to create a 'like' relation between an user and a learning object.
      # If the +user+ already liked the +object+ throw an exception.
      def like!(user, object)
        if liked? user, object
          raise 'The user already likes this object.'
        end
        like user, object
      end

      ##
      # Call this method to create a 'like' relation between an user and a learning object.
      # If the user already has the like relation, then nothing happens.
      #
      # If you want raise an exeception, use the LearningObjectRepository::like! method.
      #
      # Usage:
      #   repository.for(:objects).like current_user, @object
      def like(user, object)
        like_edge_class = 'Likes'

        # If not liked, like and set date
        if !liked? user, object
          # The properties of edge. In this case, only date for edge 'Likes'
          values = {date: sanitize_orientdb_values(Time.now)}

          create_and_set_edge like_edge_class, user.rid, object.id, values
          object.likes = object.likes + 1
        end
      end

      ##
      # Call this method checks if the +user+ has a like relation with +object+
      # This methods returns a boolean
      def liked?(user, object)
        edge_exists? 'Likes', user.rid, object.id
      end

      ##
      # Call this methods for increment the downloads number of an +object+
      # You are creating the 'downloads' relation between an +user+ and +object+
      # This relation is created only once.
      #
      # Nothing is returned
      def download(user, object)
        download_edge_class = 'Downloads'
        values = {date: sanitize_orientdb_values(Time.now)}

        create_and_set_edge download_edge_class, user.rid, object.id, values
        object.downloads = object.downloads + 1
      end

      ##
      # Call this methods for increment the views number of an +object+
      # You are creating the 'views' relation between an +user+ and +object+
      # This relation is created only once.
      #
      # Nothing is returned
      def increment_views(user, object)
        values = {date: sanitize_orientdb_values(Time.now)}

        # If doesn't exist edges from user and learning object, create it and increment attribute
        if !edge_exists? "Views", user.rid, object.id
          create_and_set_edge "Views", user.rid, object.id, values
          object.views = object.views + 1
          # If exists, update the date property of view
        else
          edge = get_edges "Views", "in", object.id
          edge = edge.first
          set_values_edge edge["@rid"], values
        end
      end

    end
  end
end
