require 'resolv-replace'

namespace :orientdb do
  desc "Orient DB integration tasks"

  task migrate: :environment do
    desc "Migrate OrientDB schema"

    migrations = OrientDb::Migrations.new(OrientDb::Client.instance)
    migrations.run
  end

  task drop: :environment do
    desc "Drop OrientDB schema"

    migrations = OrientDb::Migrations.new(OrientDb::Client.instance)
    migrations.drop
  end

  task :create_learning_object_relations, [:log, :create_first_publ, :create_first_subj, :create_first_attr] => [:environment] do |t, args|
    desc "Create LearningObject relations based on its metadata"

    include RepositoriesProxy
    include Log::Logging

    args.with_defaults(:log => STDOUT, :create_first_publ => 1, :create_first_subj => 1, :create_first_attr => 1)
    create_first_publ = (args.create_first_publ.to_i != 0)
    create_first_subj = (args.create_first_subj.to_i != 0)
    create_first_attr = (args.create_first_attr.to_i != 0)

    Log::Logging.logger = Log::DatabaseLogger.new(args.log)
    logger.level = Log::DatabaseLogger::CREATE

    helper = OrientDb::CreateRelationsHelper.new
    # Quantity of LearningObjects fetched on each iteration
    limit = 500
    # Starting point from where LearningObjects will be fetched
    offset = 5104

    loop do
      logger.info " --> Creating LearningObjects relations from #{offset} to #{offset+limit}"

      begin
        # Get LearningObjects from OrientDB (from offset to offset+limit)
        learning_objects = learning_object_repository.all_from_offset_to_limit(offset,limit)
      rescue => e
        logger.warn "Database error, going to sleep"
        logger.error e
        # Sleeps for a while to wait database's recovery
        sleep(10.seconds)
        # Goes to next iteration to retry
        next
      else
        # Terminate loop if there are no more LearningObjects
        break if learning_objects.empty?

        learning_objects.each do |lo|
          logger.info "LO ID ==== "+lo.id.to_s
          # Get metadata as a hash with metadata name as key
          metadata = helper.get_unique_metadata_keys(lo)
          # Get subjects and publishers
          subjects = metadata["dc.subject.category"]
          publisher = metadata["dc.creator"].first
          # If subjects == nil, set as empty array
          subjects ||= []
          # Remove subjects and publishers from the metadata hash, so they wont be inserted as Attributes
          metadata.delete("dc.subject.category")
          metadata.delete("dc.creator")
          # Set LearningObject relations attributes so it wont perform a search in the database when accessed
          lo.subjects=[]
          lo.attributes=[]

          unless publisher.blank?
            # Add institution on the LearningObject
            lo.publisher = helper.get_institution(publisher, create_first_publ)
          end

          # Remove nil values from subjects array
          subjects.compact!
          subjects.each do |value|
            # Split subjects names
            names = value.split("::")
            names.each_with_index do |name, i|
              # Look for subject with the same name
              subject = helper.get_subject(name, create_first_subj)
              # The subjects names are hierarchically separated by '::', so if this is not the first subject, set the previous one as supertopic of the current subject
              unless i==0 || subject.supertopics.include?(lo.subjects.last)
                subject_repository.add_supertopic(subject, lo.subjects.last)
              end
              # Add subject on the LearningObject list
              lo.subjects << subject
            end
          end

          metadata.each do |key, values|
            # Remove nil values from values array
            values.compact!
            values.each do |v|
              # Remove characters that might throw exceptions in OrientDB
              v = v.delete('\\').delete('[').delete(']')
              # Add attribute on the LearningObject list
              lo.attributes << helper.get_attribute(key, v, create_first_attr)
            end
          end
          
          # Create LearningObject relations that were added to its attrs
          learning_object_repository.create_relations(lo)
        end
        # Increase offset for next loop
        offset += limit
      end
    end
    logger.close
  end

end
